@@ -177,7 +177,7 @@ drag_data_received_callback (GtkWidget *widget,
 	if (new_windows_for_extras) {
 		int i;
 
-		application = NAUTILUS_WINDOW (window)->application;
+		application = nautilus_application_dup_singleton ();
 		screen = gtk_window_get_screen (GTK_WINDOW (window));
 
 		for (i = 1; names[i] != NULL; ++i) {
@@ -186,6 +186,8 @@ drag_data_received_callback (GtkWidget *widget,
 			nautilus_window_go_to (new_window, location);
 			g_object_unref (location);
 		}
+
+		g_object_unref (application);
 	}
 
 	g_strfreev (names);
